﻿using System.Linq;

namespace NMag.Core
{
    public class CombinedModule : Module
    {
        public CombinedModule(int number, string reservoirName, string powerPlantName, double lrwl, double hrwl, double volume, double maxDischarge, double energyEquivalent)
            : base(ModuleType.Combined, number)
        {
            Units.Add(new ReservoirUnit(this, reservoirName, lrwl, hrwl, volume));
            Units.Add(new PowerPlantUnit(this, powerPlantName, maxDischarge, energyEquivalent));
        }

        public CombinedModule(ReservoirModule reservoirModule, PowerPlantUnit powerPlant)
            : base(reservoirModule)
        {
            ModuleType = ModuleType.Combined;

            if(reservoirModule == null || powerPlant == null) { return; }

            Units.Add(reservoirModule?.Reservoir.Clone());
            Units.Add(powerPlant.Clone());
        }

        private CombinedModule(CombinedModule module)
            : base(module)
        {
            if (module != null)
            {
                Units.Add(module.Reservoir.Clone());
                Units.Add(module.PowerPlant.Clone());
            }
        }

        public ReservoirUnit Reservoir
        {
            get { return UnitHelper.GetReservoirs(Units).FirstOrDefault(); }
        }

        public PowerPlantUnit PowerPlant
        {
            get { return UnitHelper.GetPowerPlants(Units).FirstOrDefault(); }
        }

        public override Module Clone()
        {
            return new CombinedModule(this);
        }
    }
}
