﻿using System.Collections.Generic;
using System.Linq;

namespace NMag.Core
{
    public static class ModuleHelper
    {
        public static Module CreateModule(ModuleType moduleType, int number, string name, string name2 = "")
        {
            switch(moduleType)
            {
                case ModuleType.Reservoir:
                    return new ReservoirModule(number, name, 0, 0, 0);

                case ModuleType.PowerPlant:
                    return new PowerPlantModule(number, name, 0, 0);

                case ModuleType.Transfer:
                    return new TransferModule(number, name);

                case ModuleType.ControlPoint:
                    return new ControlPointModule(number, name);

                case ModuleType.Combined:
                    return new CombinedModule(number, name, name2, 0, 0, 0, 0, 0);

                default: return null;
            }
        }

        public static List<ReservoirModule> GetReservoirs(List<Module> modules)
        {
            return modules.OfType<ReservoirModule>().ToList();
        }

        public static List<PowerPlantModule> GetPowerPlants(List<Module> modules)
        {
            return modules.OfType<PowerPlantModule>().ToList();
        }

        public static List<TransferModule> GetTransfers(List<Module> modules)
        {
            return modules.OfType<TransferModule>().ToList();
        }

        public static List<ControlPointModule> GetControlPoints(List<Module> modules)
        {
            return modules.OfType<ControlPointModule>().ToList();
        }

        public static string GetModuleAddress(Module module)
        {
            if (module == null)
            {
                return "0 - System exit";
            }

            return $"{module.Number} - {module.Name}";
        }
    }
}
