﻿using System.IO;

namespace NMag.Core
{
    public static class ProjectHelper
    {
        public static Result CreateProject(string name, out Project project, bool overwrite)
        {
            project = null;

            if (name == "")
            {
                return new Result(false, "No name given.\nNo new project will be created.");
            }

            string fileName = PathFinder.GetFullFileName(PathFinder.FileType.Project, name);

            if (fileName == "")
            {
                return new Result(false, "A new project cannot be created because the project folder could not be found.");
            }

            if (File.Exists(fileName) && !overwrite)
            {
                return new Result(false, "A project with this name already exists.");
            }

            project = new Project(name);

            return new Result(true);
        }

        public static Result OpenProject(string name, out Project project)
        {
            project = null;
            return new Result(false);
        }

        public static Result SaveProject(Project project, bool overwrite)
        {
            if(File.Exists(project.FileName) && !overwrite)
            {
                return new Result(false, "A project with this name already exists.");
            }

            JSONHelper.Serialize(project, project.FileName);

            return new Result(true, "Project successfully saved.");
        }
    }
}
