﻿using System.Collections.Generic;

namespace NMag.Core
{
    public class PowerPlantUnit : Unit
    {
        public PowerPlantUnit(Module module, string name, double maxDischarge, double energyEquivalent)
            : base(UnitType.PowerPlant, module, name)
        {
            MaxDischarge = maxDischarge;
            EnergyEquivalent = energyEquivalent;
        }

        private PowerPlantUnit(PowerPlantUnit powerPlant) 
            : base (powerPlant)
        {
            if(powerPlant == null) { return; }

            MaxDischarge = powerPlant.MaxDischarge;
            EnergyEquivalent = powerPlant.EnergyEquivalent;

            UseOptionalData = powerPlant.UseOptionalData;
            NominalHead = powerPlant.NominalHead;
            IntakeLevel = powerPlant.IntakeLevel;
            TailWaterLevel = powerPlant.TailWaterLevel;
            HeadLossCoefficient = powerPlant.HeadLossCoefficient;

            LoadEfficiencyCurve = new LoadEfficiencyCurve(powerPlant.LoadEfficiencyCurve);

            UsePeakingCurve = powerPlant.UsePeakingCurve;
            PeakingCurve = new PeakingCurve(powerPlant.PeakingCurve);
        }

        // Compulsory data
        public double MaxDischarge { get; set; }                // m3/s
        public double EnergyEquivalent { get; set; }            // kwh/m3

        // Optional data
        public bool UseOptionalData { get; set; } = false;
        public double NominalHead { get; set; } = 0;            // m
        public double IntakeLevel { get; set; } = 0;            // masl
        public double TailWaterLevel { get; set; } = 0;         // masl
        public double HeadLossCoefficient { get; set; } = 0;    // s2/m5
        public LoadEfficiencyCurve LoadEfficiencyCurve { get; set; } = new LoadEfficiencyCurve(new List<Point>());      // Curve code 200

        public bool UsePeakingCurve { get; set; }
        public PeakingCurve PeakingCurve { get; set; } = new PeakingCurve(new List<Point>());   // Curve code 400

        public override Unit Clone()
        {
            return new PowerPlantUnit(this);
        }
    }
}
