﻿using System.Collections.Generic;

namespace NMag.Core
{
    public class ReservoirUnit : Unit
    {
        public ReservoirUnit(Module module, string name, double lrwl, double hrwl, double volume)
            : base(UnitType.Reservoir, module, name)
        {
            LRWL = lrwl;
            HRWL = hrwl;
            Volume = volume;
        }

        private ReservoirUnit(ReservoirUnit reservoir) 
            : base(reservoir)
        {
            if(reservoir == null) { return; }

            LRWL = reservoir.LRWL;
            HRWL = reservoir.HRWL;
            Volume = reservoir.Volume;

            UseVolumeCurve = reservoir.UseVolumeCurve;
            VolumeCurve = new VolumeCurve(reservoir.VolumeCurve);

            UseSpillFlowCurve = reservoir.UseSpillFlowCurve;
            SpillFlowCurve = new SpillFlowCurve(reservoir.SpillFlowCurve);

            OutletLevel = reservoir.OutletLevel;
            UseGateDischargeCurve = reservoir.UseGateDischargeCurve;
            GateDischargeCurve = new GateDischargeCurve(reservoir.GateDischargeCurve);

            UseEvaporationCurve = reservoir.UseEvaporationCurve;
            EvaporationCurve = new EvaporationCurve(reservoir.EvaporationCurve);
        }

        // Compulsory data
        public double LRWL { get; set; }                // masl
        public double HRWL { get; set; }                // masl
        public double Volume { get; set; }              // mill m3

        // Optional data
        public bool UseVolumeCurve { get; set; } = false;
        public VolumeCurve VolumeCurve { get; set; } = new VolumeCurve(new List<Point>());                          // Curve code 200

        public bool UseSpillFlowCurve { get; set; } = false;
        public SpillFlowCurve SpillFlowCurve  { get; set; } = new SpillFlowCurve(new List<Point>());                // Curve code 300

        public double OutletLevel { get; set; } = 0;    // masl
        public bool UseGateDischargeCurve { get; set; } = false;
        public GateDischargeCurve GateDischargeCurve { get; set; } = new GateDischargeCurve(new List<Point>());     // Curve code 400

        public bool UseEvaporationCurve { get; set; } = false;
        public EvaporationCurve EvaporationCurve { get; set; } = new EvaporationCurve(new List<Point>());           // Curve code 500

        public override Unit Clone()
        {
            return new ReservoirUnit(this);
        }
    }
}
