﻿using System.Collections.Generic;

namespace NMag.Core
{
    public class TransferUnit : Unit
    {
        public TransferUnit(Module module, string name)
            : base(UnitType.Transfer, module, name) { }

        private TransferUnit(TransferUnit transfer) 
            : base (transfer)
        {
            if(transfer == null) { return; }

            Mode = transfer.Mode;

            Capacity = transfer.Capacity;

            IntakeElevation = transfer.IntakeElevation;
            OutletElevation = transfer.OutletElevation;
            HeadLossCoefficient = transfer.HeadLossCoefficient;

            IntakeElevation = transfer.IntakeElevation;
            TailwaterElevation = transfer.TailwaterElevation;
            HQCapacityCurve = new HQCapacityCurve(transfer.HQCapacityCurve);
        }

        public enum TransferMode
        {
            None = 0,
            FixedCapacity = 100,
            VariableCapacity = 200,
            HQCapacityCurve = 300
        }

        // Optional data
        public TransferMode Mode { get; set; } = TransferMode.None;

        // Transfer mode 100
        public double Capacity { get; set; } = 0;               // m3/s

        // Transfer mode 200
        public double InletElevation { get; set; } = 0;         // masl
        public double OutletElevation { get; set; } = 0;        // masl
        public double HeadLossCoefficient { get; set; } = 0;    // s2/m5

        // Transfer mode 300
        public double IntakeElevation { get; set; } = 0;        // masl
        public double TailwaterElevation { get; set; } = 0;     // masl
        public HQCapacityCurve HQCapacityCurve { get; set; } = new HQCapacityCurve(new List<Point>());      // Curve code 300

        public override Unit Clone()
        {
            return new TransferUnit(this);
        }
    }
}
