﻿using NMag.Core.Utilization;
using System.Collections.Generic;
using System.Linq;

namespace NMag.Core
{
    public class Hydrology
    {
        private List<RunoffSeries> _runoffSeries = new List<RunoffSeries>();

        public Hydrology() { }

        private Hydrology(Hydrology hydrology)
        {
            TimeResolution = hydrology.TimeResolution;
            _runoffSeries = hydrology._runoffSeries.Select(r => r.Clone()).ToList();
        }

        public int TimeResolution { get; set; }
        //public int FirstYear
        //{
        //    get
        //    {
        //        if(RunoffSeries.Count(s => s.HydroCurve.Points.Count() >= 2) == 0) { return 0; }
        //        return RunoffSeries.Where(s => s.HydroCurve.Points.Count() >= 2)
        //            .Select(s => DateHelper.DayToDate(s.HydroCurve.FirstYear, (int)s.HydroCurve.Values[0].First()).Year).Max();
        //    }
        //}
        //public int LastYear
        //{
        //    get
        //    {
        //        if (RunoffSeries.Count(s => s.HydroCurve.Points.Count() >= 2) == 0) { return 0; }
        //        return RunoffSeries.Where(s => s.HydroCurve.Points.Count() >= 2)
        //            .Select(s => DateHelper.DayToDate(s.HydroCurve.FirstYear, (int)s.HydroCurve.Values[0].Last()).Year).Min();
        //    }
        //}

        public IReadOnlyList<RunoffSeries> RunoffSeries
        {
            get { return _runoffSeries; }
        }

        public void AddRunoffSeries(RunoffSeries series)
        {
            _runoffSeries.Add(series);
        }

        public void EditRunoffSeries(string oldName, string newName)
        {
            if (_runoffSeries.Select(r => r.Name).Contains(oldName) && _runoffSeries.Count(r => r.Name == newName) == 0)
            {
                _runoffSeries.First(r => r.Name == oldName).Name = newName;
            }
        }

        public void RemoveRunoffSeries(string name)
        {
            if (_runoffSeries.Select(r => r.Name).Contains(name))
            {
                _runoffSeries.Remove(_runoffSeries.First(r => r.Name == name));
            }
        }

        public Hydrology Clone()
        {
            return new Hydrology(this);
        }

        public Result Validate()
        {
            if(RunoffSeries.Count == 0) { return new Result(false, "No runoff data."); }

            List<string> invalidSeriesNames = RunoffSeries.Where(r => r.HydroCurve.Points.Count < 2).Select(r => r.Name).ToList();

            if(invalidSeriesNames.Count > 0)
            {
                return new Result(false, $"Runoff series {invalidSeriesNames.First()} is missing data.");
            }

            invalidSeriesNames = RunoffSeries
                .Where(r => r.HydroCurve.FirstYear == 0 || 
                    r.HydroCurve.LastYear == 0 || 
                    r.HydroCurve.FirstYear > r.HydroCurve.LastYear)
                .Select(r => r.Name)
                .ToList();

            if(invalidSeriesNames.Count > 0)
            {
                return new Result(false, $"Runoff series {invalidSeriesNames.First()} has invalid time period.");
            }

            List<int> days = RunoffSeries.First().HydroCurve.Points.Select(p => (int)p.X).ToList();

            foreach(RunoffSeries series in RunoffSeries.Skip(1))
            {
                if(!series.HydroCurve.Points.Select(p => (int)p.X).SequenceEqual(days))
                {
                    return new Result(false, $"Runoff Series {RunoffSeries.First().Name} is not compatible with runoff series {series.Name}.\nCheck the dates.");
                }
            }

            return new Result(true);
        }

        public HydroCurve GetCombinedHydroCurve(Module module)
        {
            if(!Validate().Succes || module == null)
            {
                return null;
            }

            if(module.RunoffSeriesWeights.Count == 0)
            {
                return new HydroCurve(new List<Point>(), 2000);
            }

            List<Point> points = RunoffSeries.First().HydroCurve.Points.Select(p => new Point(p.X, 0)).ToList();

            double weightSum = module.RunoffSeriesWeights.Values.Sum();

            foreach(RunoffSeries series in RunoffSeries)
            {
                if(!module.RunoffSeriesWeights.ContainsKey(series.Name))
                {
                    continue;
                }

                double factor = (module.RunoffSeriesWeights[series.Name] / weightSum) * (module.AnnualRunoff / series.GetAnnualRunoff());

                for (int p = 0; p < series.HydroCurve.Points.Count; p++)
                {
                    points[p].Y += series.HydroCurve.Points[p].Y * factor;
                }
            }

            return new HydroCurve(points, RunoffSeries.First().HydroCurve.FirstYear);
        }
    }
}