﻿using System.Collections.Generic;
using System.Linq;

namespace NMag.Core
{
    public class RunoffSeries
    {
        public RunoffSeries(string name)
        {
            Name = name;
        }

        private RunoffSeries(RunoffSeries series)
        {
            Name = series.Name;

            Frad = series.Frad;
            StationNumber = series.StationNumber;
            StationCode = series.StationCode;
            ScalingFactor = series.ScalingFactor;

            HydroCurve = new HydroCurve(series.HydroCurve);
        }

        public string Name { get; set; }

        public int Frad { get; set; } = 1;             
        public int StationNumber { get; set; } = 0;
        public string StationCode { get; set; } = "";
        public double ScalingFactor { get; set; } = 1;

        public HydroCurve HydroCurve { get; set; } = new HydroCurve(new List<Point>(), 0);

        public RunoffSeries Clone()
        {
            return new RunoffSeries(this);
        }

        public double GetAnnualRunoff()
        {
            if (HydroCurve == null || HydroCurve.Points.Count == 0)
            {
                return 0;
            }

            double years = HydroCurve.Points.Count / 365;

            return HydroCurve.Points.Select(p => p.Y).Sum() * 86400 / 1000000 / years;
        }
    }
}
