﻿using System.Collections.Generic;

namespace NMag.Core
{
    public class NMagOperationalStrategy
    {
        public NMagOperationalStrategy() { }

        private NMagOperationalStrategy(NMagOperationalStrategy strategy)
        {
            Strategy = strategy.Strategy;
            AnnualRelease = strategy.AnnualRelease;
            ReservoirReleaseCurve = new ReservoirReleaseCurve(strategy.ReservoirReleaseCurve);
            ReservoirGuideCurve = new ReservoirGuideCurve(strategy.ReservoirGuideCurve);
        }

        public enum OperationalStrategy
        {
            Automatic = 300,
            ReservoirRelease = 400,
            ReservoirGuide = 500
        }

        public OperationalStrategy Strategy { get; set; }

        public double AnnualRelease { get; set; }        // m3/s
        public ReservoirReleaseCurve ReservoirReleaseCurve { get; set; } = new ReservoirReleaseCurve(new List<Point>());    // Curve code 400
        public ReservoirGuideCurve ReservoirGuideCurve { get; set; } = new ReservoirGuideCurve(new List<Point>());          // Curve code 500

        public NMagOperationalStrategy Clone()
        {
            return new NMagOperationalStrategy(this);
        }
    }
}
