﻿using NMag.Core.Utilization;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace NMag.Core
{
    public class DataWriter
    {
        public Result Write(string path, List<string> lines, string delimiter)
        {
            return WriteFile(path, lines, delimiter);
        }

        public Result Write(string path, List<DataLine> lines, string delimiter)
        {
            return WriteFile(path, lines, delimiter);
        }

        public Result Write(string path, Curve curve, string delimiter, bool exportHeaders = true)
        {
            return WriteFile(path, GetLines(curve, exportHeaders), delimiter);
        }

        private Result WriteFile(string path, List<string> lines, string delimiter)
        {
            try
            {
                File.WriteAllLines(path, lines, Encoding.GetEncoding(1252));
            }
            catch (Exception exception)
            {
                return new Result(false, $"Could not write file.\nException: {exception}");
            }

            return new Result(true);
        }

        private Result WriteFile(string path, List<DataLine> lines, string delimiter)
        {
            try
            {
                File.WriteAllLines(path, lines
                    .Select(l => l.ToString(delimiter)), Encoding.GetEncoding(1252));
            }
            catch (Exception exception)
            {
                return new Result(false, $"Could not write file.\nException: {exception}");
            }

            return new Result(true);
        }

        private List<DataLine> GetLines(Curve curve, bool exportHeaders)
        {
            List<DataLine> lines = new List<DataLine>();

            if (exportHeaders)
            {
                lines.Add(new DataLine(curve.Axes.Select(a => a.Title).ToList()));
            }

            foreach (Core.Point point in curve.Points)
            {
                lines.Add(new DataLine(new List<string>() { Util.Format(point.X, 3), Util.Format(point.Y, 3) }));
            }

            return lines;
        }
    }
}
