﻿using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

namespace NMag.Core
{
    public static class PathFinder
    {
        private static string MAIN_FOLDER_NAME = "Data";

        public enum FileType
        {
            Project,
            DataSet,
            Results
        }

        public enum Folder
        {
            NMag,
            Projects,
            Project,
            Simulation,
            Results
        }

        public static string GetExtension(FileType fileType)
        {
            switch (fileType)
            {
                case FileType.Project:
                    return ".nmag";

                case FileType.DataSet:
                    return ".set";

                case FileType.Results:
                    return ".csv";

                default:
                    return ".txt";
            }
        }

        public static string GetFolder(Folder folder, string projectName = "", bool clear = false)
        {
            if (GetMainFolder() == "")
            {
                return "";
            }

            switch (folder)
            {
                case Folder.NMag:
                    return CheckFolder(GetMainFolder(), clear);

                case Folder.Projects:
                    return CheckFolder(Path.Combine(GetMainFolder(), folder.ToString()), clear);

                case Folder.Project:
                    return CheckFolder(Path.Combine(GetFolder(Folder.Projects, ""), projectName), clear);

                case Folder.Simulation:
                    return CheckFolder(Path.Combine(GetMainFolder(), folder.ToString()), clear);

                default:
                    return "";
            }
        }

        public static string GetFullFileName(FileType fileType, string name, string projectname = "")
        {
            if (GetMainFolder() == "")
            {
                return "";
            }
            else
            {
                string folder = fileType == FileType.Project 
                                ? GetFolder(GetFileFolder(fileType), name) 
                                : GetFolder(GetFileFolder(fileType), projectname);

                string fileName = name + GetExtension(fileType);

                return Path.Combine(folder, fileName);
            }
        }

        private static string CheckFolder(string folder, bool clear = false)
        {
            if (folder == "")
            {
                return "";
            }

            if (clear && Directory.Exists(folder))
            {
                foreach(string file in Directory.GetFiles(folder))
                {
                    try
                    {
                        File.Delete(file);
                    }
                    catch { }
                }

                foreach(string dir in Directory.GetDirectories(folder))
                {
                    try
                    {
                        Directory.Delete(dir, true);
                    }
                    catch { }
                }
            }

            if (!Directory.Exists(folder))
            {
                Directory.CreateDirectory(folder);
            }

            return folder;
        }

        private static string GetMainFolder()
        {
            return Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location), MAIN_FOLDER_NAME);
        }

        private static Folder GetFileFolder(FileType fileType)
        {
            switch (fileType)
            {
                case FileType.Project:
                case FileType.DataSet:
                case FileType.Results:

                    return Folder.Project;

                default:

                    return Folder.NMag;
            }
        }
    }
}
