﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NMag.Core.Optimization
{
    public class Generation
    {
        private PopulationControl _populationControl;
        private Action<CandidateSolution> _evaluation;

        public Generation(PopulationControl populationControl, Action<CandidateSolution> evaluation)
        {
            Birth = DateTime.Now;
            _populationControl = populationControl;
            _evaluation = evaluation;
        }

        private Generation(Generation generation)
        {
            Birth = DateTime.Now;
            Number = generation.Number + 1;
            Mutator = generation.Mutator;

            _populationControl = generation._populationControl;
            _evaluation = generation._evaluation;
        }

        public DateTime Birth { get; private set; }
        public int Number { get; private set; } = 0;
        public Random Mutator { get; private set; } = new Random();

        public List<CandidateSolution> Population = new List<CandidateSolution>();

        public CandidateSolution Fittest
        {
            get
            {
                if(Population.Count(s => !s.IsDead) == 0) { return null; }

                double maxFitness = Population.Where(s => !s.IsDead).Max(s => s.FitNess);
                return Population.First(s => s.FitNess == maxFitness);
            }
        }

        public void Populate(List<Chromosome> seeds)
        {
            Population.Clear();

            for (int p = 0; p < _populationControl.PopulationSize; p++)
            {
                CandidateSolution solution = new CandidateSolution(this, p, seeds);
                solution.Mutate(_populationControl.MutationProbability, _populationControl.MutationSpeed, _populationControl.MutationRange, Mutator);
                Population.Add(solution);
            }
        }

        public Generation GenerateOffspring()
        {
            double maxFitness = Population.Where(s => !s.IsDead).Max(s => s.FitNess);
            CandidateSolution parent = Population.First(s => s.FitNess == maxFitness);
           
            Generation newGeneration = new Generation(this);
            newGeneration.Populate(parent);
            return newGeneration;
        }

        public void EvaluateGeneration()
        {
            Population.ForEach(s => s.Evaluate(_evaluation));
        }

        private void Populate(CandidateSolution parent)
        {
            Population.Clear();

            for (int p = 0; p < _populationControl.PopulationSize; p++)
            {
                CandidateSolution solution = new CandidateSolution(this, p, parent);
                solution.Mutate(_populationControl.MutationProbability, _populationControl.MutationSpeed, _populationControl.MutationRange, Mutator);
                Population.Add(solution);
            }
        }
    }
}
