﻿using System.Collections.Generic;
using System.Linq;

namespace NMag.Core.Simulation
{
    public class ModuleResult
    {
        public ModuleResult(Module module)
        {
            Module = module.Clone();
        }

        public Module Module { get; set; }

        public List<double> Storage { get; private set; } = new List<double>();
        public List<double> WaterLevel { get; private set; } = new List<double>();

        public List<double> LocalInflow { get; private set; } = new List<double>();
        public List<double> TotalInflow { get; private set; } = new List<double>();

        public List<double> ThroughFlow { get; private set; } = new List<double>();
        public List<double> Bypass { get; private set; } = new List<double>();
        public List<double> Spill { get; private set; } = new List<double>();

        public List<double> Production { get; private set; } = new List<double>();
        
        public double LocalInflowSum
        {
            get { return LocalInflow.Sum() * 86400 / 1000000; }
        }
        public double LocalInflowAverage
        {
            get { return LocalInflow.Count == 0 ? 0 : LocalInflowSum / LocalInflow.Count * 365; }
        }

        public double TotalInflowSum
        {
            get { return TotalInflow.Sum() * 86400 / 1000000; }
        }
        public double TotalInflowAverage
        {
            get { return TotalInflow.Count == 0 ? 0 : TotalInflowSum / TotalInflow.Count * 365; }
        }

        public double ThroughFlowSum
        {
            get { return ThroughFlow.Sum() * 86400 / 1000000; }
        }
        public double ThroughFlowAverage
        {
            get { return ThroughFlow.Count == 0 ? 0 : ThroughFlowSum / ThroughFlow.Count * 365; }
        }

        public double BypassSum
        {
            get { return Bypass.Sum() * 86400 / 1000000; }
        }
        public double BypassAverage
        {
            get { return Bypass.Count == 0 ? 0 : BypassSum / Bypass.Count * 365; }
        }

        public double SpillSum
        {
            get { return Spill.Sum() * 86400 / 1000000; }
        }
        public double SpillAverage
        {
            get { return Spill.Count == 0 ? 0 : SpillSum / Spill.Count * 365; }
        }

        public double ProductionSum
        {
            get { return Production.Sum() * 24 / 1000; }
        }
        public double ProductionAverage
        {
            get { return Production.Count == 0 ? 0 : ProductionSum / Production.Count * 365; }
        }
    }
}
