﻿using NMag.Core;
using NMag.Core.Utilization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;

namespace NMag.UI.Dialogs
{
    /// <summary>
    /// Interaction logic for DateInputDialog.xaml
    /// </summary>
    public partial class DateInputDialog : BaseDialog
    {
        private int _day = 1;
        private int _month = 1;

        private int _firstYear = 0;

        public DateInputDialog(int firstYear)
        {
            InitializeComponent();

            UpdateMonths();

            DayNumber = 1;

            if(firstYear > 0)
            {
                _firstYear = firstYear;
                Label_Year.Visibility = Visibility.Visible;
                Text_Year.Visibility = Visibility.Visible;
                Text_Year.IsEnabled = true;
            }
        }

        public int DayNumber
        {
            get
            {
                if (_firstYear == 0)
                {
                    return DateHelper.DayOfYear(_month, _day);
                }

                return DateHelper.DayOfYear(_firstYear, new DateTime((int)Text_Year.Value, _month, _day));
            }
            set
            {
                if (value < 1)
                {
                    return;
                }

                if (value > 365 && _firstYear == 0)
                {
                    return;
                }

                DateTime date = DateHelper.DayToDate(1999, value);

                if (_firstYear > 0)
                {
                    date = DateHelper.DayToDate(_firstYear, value);

                    Text_Year.Value = date.Year;
                }
                
                ComboBox_Month.SelectedIndex = date.Month - 1;
                ComboBox_Day.SelectedIndex = date.Day - 1;
            }
        }

        private int DaysInMonth
        {
            get
            {
                return DateTime.DaysInMonth(1999, _month);
            }
        }

        private void UpdateMonths()
        {
            ComboBox_Month.Items.Clear();

            for(int m = 1; m <= 12; m++)
            {
                ComboBox_Month.Items.Add(new DateTime(1999, m, 1).ToString("MMMM"));
            }

            ComboBox_Month.SelectedIndex = _month - 1;
        }

        private void UpdateDays(int selectedDay)
        {
            ComboBox_Day.Items.Clear();

            for(int i = 1; i <= DaysInMonth; i++)
            {
                ComboBox_Day.Items.Add(i.ToString());
            }

            ComboBox_Day.SelectedIndex = selectedDay - 1;
        }

        private void ComboBox_Day_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            _day = Math.Max(1, ComboBox_Day.SelectedIndex + 1);
        }

        private void ComboBox_Month_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            _month = Math.Max(1, ComboBox_Month.SelectedIndex + 1);

            UpdateDays(Math.Min(ComboBox_Day.SelectedIndex + 1, DaysInMonth));
        }

        private void Button_Ok_Click(object sender, RoutedEventArgs e)
        {
            On_Ok();
        }

        private void Button_Cancel_Click(object sender, RoutedEventArgs e)
        {
            On_Cancel();
        }

        private void Text_Year_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            if(Text_Year.Value > 9999)
            {
                Text_Year.Value = 9999;
            }

            if (Text_Year.Value < 1900)
            {
                Text_Year.Value = 1900;
            }
        }
    }
}
