﻿using NMag.Core;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Shapes;

namespace NMag.UI.FlowDiagram
{
    public class ModuleModel
    {
        private ModuleControlFactory _moduleFactory;

        private List<ModuleControl> _moduleControls = new List<ModuleControl>();

        public ModuleModel(MouseButtonEventHandler onModuleClickHandler)
        {
            _moduleFactory = new ModuleControlFactory(onModuleClickHandler);
        }

        public void UpdateModules(List<Module> modules)
        {
            _moduleControls = _moduleFactory.CreateModuleControls(modules).ToList();
        }

        public int[] GetModuleBoundaries()
        {
            int[] boundaries = new int[4];
            boundaries[0] = (int)_moduleControls.Select(c => c.Shapes.Select(s => s.Position.X).Min()).Min();
            boundaries[1] = (int)_moduleControls.Select(c => c.Shapes.Select(s => s.Position.Y).Max()).Max();
            boundaries[2] = (int)_moduleControls.Select(c => c.Shapes.Select(s => s.Position.X).Max()).Max();
            boundaries[3] = (int)_moduleControls.Select(c => c.Shapes.Select(s => s.Position.Y).Min()).Min();

            return boundaries;
        }

        internal IEnumerable<object> GetConnections()
        {
            foreach (ModuleControl moduleControl in _moduleControls.Where(m => m != null))
            {
                foreach (Line connection in moduleControl.Connections)
                {
                    yield return connection;
                }
            }
        }

        public IEnumerable<ShapeBase> GetShapes()
        {
            foreach (ModuleControl moduleControl in _moduleControls.Where(m => m != null))
            {
                foreach (ShapeBase shape in moduleControl.Shapes)
                {
                    yield return shape;
                }
            }
        }

        public IEnumerable<Border> GetTextBlocks()
        {
            foreach (ModuleControl moduleControl in _moduleControls.Where(m => m != null))
            {
                foreach (Label label in moduleControl.Labels)
                {
                    yield return label;
                }
            }
        }
    }
}
