﻿using NMag.Core;
using NMag.UI.CustomControls;
using NMag.UI.Models;
using System;
using System.Collections.ObjectModel;
using System.Linq;

namespace NMag.UI.Views
{
    public class EnergyMarketViewModel : MainViewModel
    {
        private string _firmPowerLevel;
        private string _firmPowerPrice;
        private string _summerStart;
        private string _summerEnd;

        private int _distributionMode;

        public EnergyMarketViewModel(ApplicationViewModel application)
            :base(application, "Energy Market", ViewModelGroup.Setup)
        {
            GetData();
        }

        public override ToolbarViewModel Toolbar
        {
            get { return new EnergyMarketToolbarViewModel(this); }
        }

        public string FirmPowerLevel
        {
            get { return _firmPowerLevel; }
            set { _firmPowerLevel = value; }
        }
        public string FirmPowerPrice
        {
            get { return _firmPowerPrice; }
            set { _firmPowerPrice = value; }
        }
        public string SummerStart
        {
            get { return _summerStart; }
            set { _summerStart = value; }
        }
        public string SummerEnd
        {
            get { return _summerEnd; }
            set { _summerEnd = value; }
        }

        public int DistributionMode
        {
            get { return _distributionMode; }
            set
            {
                _distributionMode = value;
                Update();
            }
        }

        public bool UseDailyLoadDistribution
        {
            get { return _distributionMode == (int)EnergyMarket.DistributionMode.DailyLoad; }
        }
        public ChartViewModel DailyLoadChart
        {
            get
            {
                if(Market?.DailyLoadCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        Market.DailyLoadCurve, 
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public bool UseVariableDistribution
        {
            get { return _distributionMode == (int)EnergyMarket.DistributionMode.Vairable; }
        }
        public ChartViewModel VariableLoadChart
        {
            get
            {
                if (Market?.VariableLoadCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        Market.VariableLoadCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public string PreferenceCurvesInvalidText
        {
            get { return Market?.ValidateCurves().Message ?? ""; }
        }

        public ObservableCollection<PreferenceFunctionModel> PreferenceFunctions
        {
            get
            {
                if(Market == null) { return null; }

                return new ObservableCollection<PreferenceFunctionModel>(Market.PreferenceFunctions
                    .Select(p => new PreferenceFunctionModel(p, Update)));
            }
        }
        public PreferenceFunctionModel SelectedFunction { get; set; }

        private EnergyMarket Market
        {
            get { return Application.Project?.EnergyMarket; }
        }

        public void AddPreferenceFunction()
        {
            Market.PreferenceFunctions.Add(new PreferenceFunction());
            Update();
        }

        public void DeletePreferenceFunction()
        {
            if(!PreferenceFunctions.Contains(SelectedFunction)) { return; }

            Market.PreferenceFunctions.Remove(SelectedFunction.PreferenceFunction);
            Update();
        }

        public override void SaveData()
        {
            if(Market == null) { return; }

            if (double.TryParse(FirmPowerLevel, out double doubleValue)) { Market.FirmPowerLevel = doubleValue; }
            if (double.TryParse(FirmPowerPrice, out doubleValue)) { Market.FirmPowerPrice = doubleValue; }
            if (int.TryParse(SummerStart, out int intValue)) { Market.SummerStart = intValue; }
            if (int.TryParse(SummerEnd, out intValue)) { Market.SummerEnd = intValue; }

            Market.Mode = (EnergyMarket.DistributionMode)DistributionMode;
        }

        public override void GetData()
        {
            if (Market == null) { return; }

            FirmPowerLevel = Market.FirmPowerLevel.ToString();
            FirmPowerPrice = Market.FirmPowerPrice.ToString();
            SummerStart = Market.SummerStart.ToString();
            SummerEnd = Market.SummerEnd.ToString();

            DistributionMode = (int)Market.Mode;
        }

        public override void Update()
        {
            UpdateToolbar();

            OnPropertyChanged("FirmPowerLevel");
            OnPropertyChanged("FirmPowerPrice");
            OnPropertyChanged("SummerStart");
            OnPropertyChanged("SummerEnd");

            OnPropertyChanged("DistributionMode");

            OnPropertyChanged("UseDailyLoadDistribution");
            OnPropertyChanged("DailyLoadChart");

            OnPropertyChanged("UseVariableDistribution");
            OnPropertyChanged("VariableLoadChart");

            OnPropertyChanged("PreferenceFunctions");
            OnPropertyChanged("PreferenceCurvesInvalidText");
        }
    }
}
