﻿using NMag.Core;
using NMag.UI.CustomControls;
using NMag.UI.Miscellaneous;
using NMag.UI.Windows;
using System.Windows.Input;

namespace NMag.UI.Views
{
    public class HydrologySelectionViewModel : ModuleDataViewModel
    {
        private string _annualRunoff;

        public HydrologySelectionViewModel(ModuleSetupViewModel moduleViewModel)
            :base (moduleViewModel)
        {
            GetData();
        }

        public bool UseLocalInflow
        {
            get { return _moduleViewModel.SelectedModule?.UseLocalInflow ?? false; }
            set
            {
                if (_moduleViewModel.SelectedModule == null) { return; }

                _moduleViewModel.SelectedModule.UseLocalInflow = value;
                Update();
            }
        }

        public string AnnualRunoff
        {
            get { return _annualRunoff; }
            set { _annualRunoff = value; }
        }

        public bool IsHydrologyValid
        {
            get { return _moduleViewModel.WaterSystem.Application.Project?.Hydrology.Validate().Succes ?? false; }
        }
        public string InvalidHydrologyText
        {
            get { return "Invalid hydrological data!"; }
        }

        public ChartViewModel HydroChart
        {
            get
            {
                HydroCurve hydroCurve = _moduleViewModel.WaterSystem.Application.Project?.Hydrology.GetCombinedHydroCurve(_moduleViewModel.SelectedModule);

                if (hydroCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        hydroCurve,
                        SeriesData.SeriesType.Line));
            }
        }

        public ICommand SelectedHydrologyCommand
        {
            get
            {
                return new RelayCommand(args => new HydrologySelectionWindow(_moduleViewModel.SelectedModule, Update).ShowDialog(),
                    args => _moduleViewModel.SelectedModule != null);
            }
        }

        public override void GetData()
        {
            if(_moduleViewModel.SelectedModule == null) { return; }

            AnnualRunoff = _moduleViewModel.SelectedModule.AnnualRunoff.ToString();

            Update();
        }

        public override void SaveData()
        {
            if (_moduleViewModel.SelectedModule == null) { return; }

            if (double.TryParse(_annualRunoff, out double value)){ _moduleViewModel.SelectedModule.AnnualRunoff = value; }
        }

        public override void Update()
        {
            OnPropertyChanged("UseLocalInflow");

            OnPropertyChanged("AnnualRunoff");
            OnPropertyChanged("IsHydrologyValid");
            OnPropertyChanged("InvalidHydrologyText");

            OnPropertyChanged("HydroChart");
        }
    }
}
