﻿using NMag.Core;
using NMag.UI.CustomControls;

namespace NMag.UI.Views
{
    public class EnMagStrategyViewModel : OperationalStrategyViewModel
    {
        private int _strategy;

        public EnMagStrategyViewModel(ModuleSetupViewModel moduleViewModel)
            : base(moduleViewModel)
        {
            GetData();
        }

        public int Strategy
        {
            get { return _strategy; }
            set
            {
                _strategy = value;
                
                Update();
            }
        }

        public bool UseGuideCurve
        {
            get { return Strategy == (int)EnMagOperationalStrategy.OperationalStrategy.GuideCurve; }
        }
        public ChartViewModel GuideChart
        {
            get
            {
                if (OperationalStrategy?.GuideCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        OperationalStrategy.GuideCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public bool UseRuleCurves
        {
            get { return Strategy == (int)EnMagOperationalStrategy.OperationalStrategy.RuleCurve; }
        }
        public ChartViewModel RationingRuleChart
        {
            get
            {
                if (OperationalStrategy?.RationingRuleCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        OperationalStrategy?.RationingRuleCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }
        public ChartViewModel SurplusRuleChart
        {
            get
            {
                if (OperationalStrategy?.SurplusRuleCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        OperationalStrategy?.SurplusRuleCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        private EnMagOperationalStrategy OperationalStrategy
        {
            get { return _moduleViewModel.SelectedModule?.EnMagOperationalStrategy; }
        }

        public override void GetData()
        {
            if(OperationalStrategy == null) { return; }

            Strategy = (int)OperationalStrategy.Strategy;
        }

        public override void SaveData()
        {
            if (OperationalStrategy == null) { return; }

            OperationalStrategy.Strategy = (EnMagOperationalStrategy.OperationalStrategy)Strategy;
        }

        public override void Update()
        {
            OnPropertyChanged("Strategy");

            OnPropertyChanged("UseGuideCurve");
            OnPropertyChanged("GuideChart");

            OnPropertyChanged("UseRuleCurves");
            OnPropertyChanged("RationingRuleChart");
            OnPropertyChanged("SurplusRuleChart");
        }
    }
}
