﻿using NMag.Core;
using NMag.UI.CustomControls;

namespace NMag.UI.Views
{
    public class RestrictionViewModel : ModuleDataViewModel
    {
        public RestrictionViewModel(ModuleSetupViewModel moduleViewModel)
            : base(moduleViewModel) { }

        public bool IsReservoir
        {
            get { return _moduleViewModel.SelectedModule == null ? false : _moduleViewModel.SelectedModule.ModuleType == ModuleType.Reservoir; }
        }

        public bool UseMinimumFlow
        {
            get { return _moduleViewModel.SelectedModule?.UseMinimumFlow ?? false; }
            set
            {
                if(_moduleViewModel.SelectedModule == null) { return; }

                _moduleViewModel.SelectedModule.UseMinimumFlow = value;
                OnPropertyChanged("UseMinimumFlow");
                OnPropertyChanged("MinimumFlowChart");
            }
        }
        public ChartViewModel MinimumFlowChart
        {
            get
            {
                if (_moduleViewModel?.SelectedModule?.MinimumFlowCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _moduleViewModel.SelectedModule?.MinimumFlowCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public bool UseBypassRelease
        {
            get { return _moduleViewModel.SelectedModule?.UseBypassRelease ?? false; }
            set
            {
                if (_moduleViewModel.SelectedModule == null) { return; }

                _moduleViewModel.SelectedModule.UseBypassRelease = value;
                OnPropertyChanged("UseBypassRelease");
                OnPropertyChanged("BypassReleaseChart");
            }
        }
        public ChartViewModel BypassReleaseChart
        {
            get
            {
                if (_moduleViewModel?.SelectedModule?.BypassReleaseCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _moduleViewModel.SelectedModule.BypassReleaseCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public bool UseMinimumLevel
        {
            get { return _moduleViewModel.SelectedModule == null ? false : _moduleViewModel.SelectedModule.UseMinimumLevel && IsReservoir; }
            set
            {
                if (_moduleViewModel.SelectedModule == null) { return; }

                _moduleViewModel.SelectedModule.UseMinimumLevel = value;
                OnPropertyChanged("UseMinimumLevel");
                OnPropertyChanged("MinimumLevelChart");
            }
        }
        public ChartViewModel MinimumLevelChart
        {
            get
            {
                if (_moduleViewModel?.SelectedModule?.MinimumLevelCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _moduleViewModel.SelectedModule.MinimumLevelCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public bool UseMaximumLevel
        {
            get { return _moduleViewModel.SelectedModule == null ? false : _moduleViewModel.SelectedModule.UseMaximumLevel && IsReservoir; }
            set
            {
                if (_moduleViewModel.SelectedModule == null) { return; }

                _moduleViewModel.SelectedModule.UseMaximumLevel = value;
                OnPropertyChanged("UseMaximumLevel");
                OnPropertyChanged("MaximumLevelChart");
            }
        }
        public ChartViewModel MaximumLevelChart
        {
            get
            {
                if (_moduleViewModel?.SelectedModule?.MaximumLevelCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _moduleViewModel.SelectedModule?.MaximumLevelCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }
        
        public override void GetData()
        {

        }

        public override void SaveData()
        {
            
        }

        public override void Update()
        {
            OnPropertyChanged("UseMinimumFlow");
            OnPropertyChanged("MinimumFlowChart");

            OnPropertyChanged("UseBypassRelease");
            OnPropertyChanged("BypassReleaseChart");

            OnPropertyChanged("UseMinimumLevel");
            OnPropertyChanged("MinimumLevelChart");

            OnPropertyChanged("UseMaximumLevel");
            OnPropertyChanged("MaximumLevelChart");
        }
    }
}
