﻿using NMag.Core;
using NMag.UI.CustomControls;

namespace NMag.UI.Views
{
    public class ReservoirViewModel : ViewModel
    {
        private ReservoirUnit _reservoir;

        private string _lrwl;
        private string _hrwl;
        private string _volume;

        private bool _useVolumeCurve;
        private bool _useSpillFlowCurve;
        private bool _useGateDischargeCurve;
        private bool _useEvaporationCurve;

        private string _outletLevel;

        public ReservoirViewModel(ReservoirUnit reservoir)
        {
            _reservoir = reservoir;

            GetData();
        }

        public string LRWL
        {
            get { return _lrwl; }
            set { _lrwl = value; }
        }
        public string HRWL
        {
            get { return _hrwl; }
            set { _hrwl = value; }
        }
        public string Volume
        {
            get { return _volume; }
            set { _volume = value; }
        }

        public bool UseVolumeCurve
        {
            get { return _useVolumeCurve; }
            set
            {
                _useVolumeCurve = value;
                OnPropertyChanged("UseVolumeCurve");
                OnPropertyChanged("VolumeChart");
            }
        }
        public ChartViewModel VolumeChart
        {
            get
            {
                if (_reservoir?.VolumeCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _reservoir?.VolumeCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }

        public bool UseSpillFlowCurve
        {
            get { return _useSpillFlowCurve; }
            set
            {
                _useSpillFlowCurve = value;
                OnPropertyChanged("UseSpillFlowCurve");
                OnPropertyChanged("SpillFlowChart");
            }
        }
        public ChartViewModel SpillFlowChart
        {
            get
            {
                if (_reservoir?.SpillFlowCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _reservoir?.SpillFlowCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }

        public string OutletLevel
        {
            get { return _outletLevel; }
            set { _outletLevel = value; }
        }
        public bool UseGateDischargeCurve
        {
            get { return _useGateDischargeCurve; }
            set
            {
                _useGateDischargeCurve = value;
                OnPropertyChanged("UseGateDischargeCurve");
                OnPropertyChanged("GateDischargeChart");
            }
        }
        public ChartViewModel GateDischargeChart
        {
            get
            {
                if (_reservoir?.GateDischargeCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _reservoir?.GateDischargeCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }
        
        public bool UseEvaporationCurve
        {
            get { return _useEvaporationCurve; }
            set
            {
                _useEvaporationCurve = value;
                OnPropertyChanged("UseEvaporationCurve");
                OnPropertyChanged("EvaporationChart");
            }
        }
        public ChartViewModel EvaporationChart
        {
            get
            {
                if (_reservoir?.EvaporationCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _reservoir?.EvaporationCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }

        public override void SaveData()
        {
            if(_reservoir == null) { return; }

            if (double.TryParse(LRWL, out double value)) { _reservoir.LRWL = value; }
            if (double.TryParse(HRWL, out value)) { _reservoir.HRWL = value; }
            if (double.TryParse(Volume, out value)) { _reservoir.Volume = value; }

            if (double.TryParse(OutletLevel, out value)) { _reservoir.OutletLevel = value; }

            _reservoir.UseVolumeCurve = UseVolumeCurve;
            _reservoir.UseSpillFlowCurve = UseSpillFlowCurve;
            _reservoir.UseGateDischargeCurve = UseGateDischargeCurve;
            _reservoir.UseEvaporationCurve = UseEvaporationCurve;
        }

        public override void GetData()
        {
            if (_reservoir == null) { return; }

            LRWL = _reservoir.LRWL.ToString();
            HRWL = _reservoir.HRWL.ToString();
            Volume = _reservoir.Volume.ToString();

            OutletLevel = _reservoir.OutletLevel.ToString();

            UseVolumeCurve = _reservoir.UseVolumeCurve;
            UseSpillFlowCurve = _reservoir.UseSpillFlowCurve;
            UseGateDischargeCurve = _reservoir.UseGateDischargeCurve;
            UseEvaporationCurve = _reservoir.UseEvaporationCurve;

            Update();
        }

        public override void Update()
        {
            OnPropertyChanged("LRWL");
            OnPropertyChanged("HRWL");
            OnPropertyChanged("Volume");
            OnPropertyChanged("OutletLevel");

            OnPropertyChanged("UseVolumeCurve");
            OnPropertyChanged("VolumeChart");

            OnPropertyChanged("UseSpillFlowCurve");
            OnPropertyChanged("SpillFlowChart");

            OnPropertyChanged("UseGateDischargeCurve");
            OnPropertyChanged("GateDischargeChart");

            OnPropertyChanged("UseEvaporationCurve");
            OnPropertyChanged("EvaporationChart");
        }
    }
}
