﻿using NMag.Core;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace NMag.UI.Views
{
    public abstract class ModuleViewModel : ViewModel
    {
        private string _selectedModuleType = "All";
        private Module _selectedModule;
        private Unit _selectedUnit;

        public ModuleViewModel(WaterSystemViewModel waterSystem)
        {
            WaterSystem = waterSystem;
        }

        public WaterSystemViewModel WaterSystem { get; private set; }

        public ObservableCollection<string> ModuleTypes
        {
            get
            {
                return WaterSystem.Application.Project == null ?
                    null :
                    new ObservableCollection<string>(new List<string>() { "All" }
                        .Concat(WaterSystem.Application.Project.Modules
                            .Select(m => m.TypeString)
                            .Distinct())
                        .ToList());
            }
        }
        public string SelectedModuleType
        {
            get { return _selectedModuleType; }
            set
            {
                _selectedModuleType = value;
                OnPropertyChanged("AllModules");
                OnPropertyChanged("SelectedModuleType");
            }
        }

        public List<string> AllModules
        {
            get
            {
                return WaterSystem.Application.Project?.Modules
                        .Where(m => _selectedModuleType == "All" ||
                            m.TypeString == _selectedModuleType.Replace(" ", ""))
                        .OrderBy(m => m.Number)
                        .Select(m => ModuleHelper.GetModuleAddress(m))
                        .ToList();
            }
        }
        public string SelectedName
        {
            get { return ModuleHelper.GetModuleAddress(SelectedModule); }
            set
            {
                if ((value ?? "") != "")
                {
                    int.TryParse(value.Split(' ').First(), out int moduleNumber);

                    SelectedModule = WaterSystem.Application.Project.Modules.FirstOrDefault(m => m.Number == moduleNumber);
                }
            }
        }
        public Module SelectedModule
        {
            get { return _selectedModule; }
            set
            {
                if (value == null) { return; }

                SaveData();
                SelectedModuleType = ModuleTypes.FirstOrDefault();
                _selectedModule = value;
                SelectedUnit = value.Units.FirstOrDefault();
                GetData();
                Update();
            }
        }

        public Unit SelectedUnit
        {
            get { return _selectedUnit; }
            set
            {
                if (value == null) { return; }
                _selectedUnit = value;
                OnPropertyChanged("DataViewModel");
            }
        }
        public string SelectedUnitType
        {
            get
            {
                if (SelectedUnit == null) { return "-1"; }
                return ((int)SelectedUnit.UnitType).ToString();
            }
            set { SelectedUnit = SelectedModule?.Units.FirstOrDefault(u => ((int)u.UnitType).ToString() == value); }
        }
        // TODO simplify
        public bool HasReservoirUnit
        {
            get
            {
                if (SelectedModule == null) { return false; }
                return SelectedModule.ModuleType == ModuleType.Reservoir || SelectedModule?.ModuleType == ModuleType.Combined;
            }
        }
        public bool HasPowerPlantUnit
        {
            get
            {
                if (SelectedModule == null) { return false; }
                return SelectedModule.ModuleType == ModuleType.PowerPlant || SelectedModule?.ModuleType == ModuleType.Combined;
            }
        }
        public bool HasTransferUnit
        {
            get
            {
                if (SelectedModule == null) { return false; }
                return SelectedModule.ModuleType == ModuleType.Transfer;
            }
        }
        public bool HasControlPointUnit
        {
            get
            {
                if (SelectedModule == null) { return false; }
                return SelectedModule.ModuleType == ModuleType.ControlPoint;
            }
        }

        public bool IsModuleSelected
        {
            get { return SelectedModule != null; }
        }
        public bool IsReservoirModule
        {
            get
            {
                if (SelectedModule == null) { return false; }

                return SelectedModule.ModuleType == ModuleType.Reservoir ||
                    SelectedModule.ModuleType == ModuleType.Combined;
            }
        }
    }
}
