﻿using NMag.Core;
using NMag.Core.Utilization;
using NMag.UI.Miscellaneous;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Input;
using System.Windows.Forms;
using System;
using System.Windows;

namespace NMag.UI.Windows
{
    public class DataExporterViewModel : WindowViewModel
    {
        private Curve _curve;

        private string _selectedFileType;
        private string _selectedPath;

        public DataExporterViewModel(EventHandler requestCloseHandler, Curve curve)
            :base (requestCloseHandler)
        {
            _curve = curve;

            SelectedDelimiter = Delimiters.First();
        }

        public enum Delimiter
        {
            Tab,
            Comma,
            Semicolon
        }
        public List<string> Delimiters
        {
            get
            {
                return Enum.GetNames(typeof(Delimiter)).ToList();
            }
        }
        public string SelectedDelimiter
        {
            get
            {
                return _selectedFileType;
            }
            set
            {
                _selectedFileType = value;
                SelectedPath = "";
                OnPropertyChanged("SelectedDelimiter");
            }
        }

        public string SelectedPath
        {
            get
            {
                return _selectedPath;
            }
            set
            {
                if((value ?? "") != "")
                {
                    _selectedPath = Path.Combine(Path.GetDirectoryName(value), 
                        Path.GetFileNameWithoutExtension(value) + GetFileFilter().Right(4));

                    OnPropertyChanged("SelectedPath");
                }
            }
        }

        public bool ExportHeaders { get; set; } = true;

        public ICommand SelectPathCommand
        {
            get
            {
                return new RelayCommand(args => SelectPath());
            }
        }
        public ICommand ExportCommand
        {
            get
            {
                return new RelayCommand(args => Export(),
                    args => CanExport());
            }
        }

        private void SelectPath()
        {
            SaveFileDialog dialog = new SaveFileDialog()
            {
                Filter = GetFileFilter()
            };
            
            if(dialog.ShowDialog() == DialogResult.OK)
            {
                SelectedPath = dialog.FileName;
            }
        }

        private bool CanExport()
        {
            return !string.IsNullOrEmpty(SelectedPath) && Directory.Exists(Path.GetDirectoryName(SelectedPath));
        }

        private void Export()
        {
            if(!CanExport())
            {
                System.Windows.MessageBox.Show("Cannot find the selected folder.", "Folder Not Found", MessageBoxButton.OK, MessageBoxImage.Error);
            }

            switch (SelectedDelimiter)
            {
                case "Comma":
                case "Semicolon":
                case "Tab":
                    new DataWriter().Write(SelectedPath, _curve, GetDelimiter(), ExportHeaders);
                    break;

                case "Text":
                    new DataWriter().Write(SelectedPath, _curve, GetDelimiter(), ExportHeaders);
                    break;

                default:
                    break;
            }

            Close();
        }

        private string GetDelimiter()
        {
            switch(SelectedDelimiter)
            {
                case "Comma":
                    return ",";

                case "Semicolon":
                    return ";";

                case "Tab":
                    return "\t";

                default:
                    return "";
            }
        }

        private string GetFileFilter()
        {
            switch (SelectedDelimiter)
            {
                case "Comma":
                    return "CSV Files (*.csv)|*.csv";
                    
                default:
                    return "";
            }
        }

        public override void Update()
        {
        }
    }
}