﻿using NMag.Core;
using NMag.UI.Miscellaneous;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

namespace NMag.UI.Windows
{
    public class HydrologySelectionViewModel : WindowViewModel
    {
        private Module _module;

        private Action _updateAction;

        private List<RunoffWeight> _runoffWeights = new List<RunoffWeight>();

        public HydrologySelectionViewModel(EventHandler requestCloseHandler, Module module, Action updateAction)
            : base(requestCloseHandler)
        {
            _updateAction = updateAction;
            _module = module;

            foreach (KeyValuePair<string, double> selection in _module.RunoffSeriesWeights)
            {
                _runoffWeights.Add(new RunoffWeight(selection.Key, selection.Value));
            }
        }

        public ICommand OkCommand
        {
            get
            {
                return new RelayCommand(args => UpdateModuleRunoffSeriesWeights());
            }
        }

        public ICommand CancelCommand
        {
            get
            {
                return new RelayCommand(args => Close());
            }
        }

        public ObservableCollection<RunoffWeight> RunoffWeights
        {
            get
            {
                return new ObservableCollection<RunoffWeight>(_runoffWeights);
            }
        }

        private void UpdateModuleRunoffSeriesWeights()
        {
            foreach(RunoffWeight selection in RunoffWeights)
            {
                if(!_module.RunoffSeriesWeights.ContainsKey(selection.Name))
                {
                    _module.RunoffSeriesWeights.Add(selection.Name, 0);
                }

                _module.RunoffSeriesWeights[selection.Name] = selection.Weight;
            }

            _updateAction.Invoke();
            Close();
        }

        public override void Update()
        {
        }

        public class RunoffWeight
        {
            public RunoffWeight(string name, double weight)
            {
                Name = name;
                Weight = weight;
            }

            public string Name { get; set; }
            public double Weight { get; set; }
        }
    }
}
